<?php
declare(strict_types=1);

namespace Flat101\Base\Service;

use Magento\Catalog\Api\CategoryRepositoryInterface;
use Magento\Catalog\Api\Data\CategoryInterface;
use Magento\Catalog\Model\Session as CatalogSession;
use Magento\Framework\App\Request\Http;
use Magento\Framework\Exception\NoSuchEntityException;

/**
 *  GetCurrentCategoryService
 */
class GetCurrentCategoryService
{
    /** @var CategoryInterface */
    private $currentCategory;

    /** @var int|null */
    private $categoryId;

    /** @var CatalogSession */
    private $catalogSession;

    /** @var CategoryRepositoryInterface */
    private $categoryRepository;

    /** @var Http */
    private $request;

    /**
     * @param CatalogSession $catalogSession
     * @param CategoryRepositoryInterface $categoryRepository
     * @param Http $request
     */
    public function __construct(
        CatalogSession $catalogSession,
        CategoryRepositoryInterface $categoryRepository,
        Http $request
    ) {
        $this->catalogSession = $catalogSession;
        $this->categoryRepository = $categoryRepository;
        $this->request = $request;
    }

    public function getCategoryId()
    {
        if ($this->request->getFullActionName() == 'catalog_category_view') {
            if (!$this->categoryId) {
                $currentCategoryId = $this->catalogSession->getData('last_viewed_category_id');

                if ($currentCategoryId) {
                    $this->categoryId = (int)$currentCategoryId;
                }
            }
        }

        return $this->categoryId;
    }

    /**
     * @return CategoryInterface|null
     */
    public function getCategory(): ?CategoryInterface
    {
        if ($this->request->getFullActionName() == 'catalog_category_view') {
            if (!$this->currentCategory) {
                $categoryId = $this->getCategoryId();

                if (!$categoryId) {
                    return null;
                }

                try {
                    $this->currentCategory = $this->categoryRepository->get($categoryId);
                } catch (NoSuchEntityException $e) {
                    return null;
                }
            }
        }

        return $this->currentCategory;
    }
}
