<?php
declare(strict_types=1);

namespace Flat101\Base\Service;

use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Catalog\Model\Session as CatalogSession;
use Magento\Framework\App\Request\Http;
use Magento\Framework\Exception\NoSuchEntityException;

/**
 *  CurrentProductDataProvider
 */
class GetCurrentProductService
{
    /** @var ProductInterface */
    private $currentProduct;

    /** @var int|null */
    private $productId;

    /** @var CatalogSession */
    private $catalogSession;

    /** @var ProductRepositoryInterface */
    private $productRepository;

    /** @var Http */
    private $request;

    /**
     * @param CatalogSession $catalogSession
     * @param ProductRepositoryInterface $productRepository
     * @param Http $request
     */
    public function __construct(
        CatalogSession $catalogSession,
        ProductRepositoryInterface $productRepository,
        Http $request
    ) {
        $this->catalogSession = $catalogSession;
        $this->productRepository = $productRepository;
        $this->request = $request;
    }

    /**
     * @return int|null
     */
    public function getProductId(): ?int
    {
        if ($this->request->getFullActionName() == 'catalog_product_view') {
            if (!$this->productId) {
                $productId = $this->catalogSession->getData('last_viewed_product_id');
                $this->productId = $productId ? (int)$productId : null;
            }
        }

        return $this->productId;
    }

    /**
     * @return ProductInterface|null
     */
    public function getProduct(): ?ProductInterface
    {
        if ($this->request->getFullActionName() == 'catalog_product_view') {
            if (!$this->currentProduct) {
                $productId = $this->getProductId();

                if (!$productId) {
                    return null;
                }

                try {
                    $this->currentProduct =  $this->productRepository->getById($this->getProductId());
                } catch (NoSuchEntityException $e) {
                    return null;
                }
            }
        }

        return $this->currentProduct;
    }
}
